﻿package MT::Plugin::RebuildAt1stView;
use strict;
use MT;
use MT::Entry;
use MT::Comment;
use MT::TBPing;
use MT::Trackback;
use MT::Plugin;
use MT::Permalink;
require MT::Plugin;

use vars qw( $VERSION $SCHEMA_VERSION );
$VERSION = '1.0b01';
$SCHEMA_VERSION = '0.50';

use base qw( MT::Plugin );

###################################### Init Plugin #####################################

@MT::Plugin::RebuildAt1stView::ISA = qw(MT::Plugin);

my $plugin = new MT::Plugin::RebuildAt1stView({
	name => 'RebuildAt1stView',
	author_name => 'Junnama Noda',
	author_link => 'http://junnama.alfasado.net/online/',
	'version' => $VERSION,
	'schema_version' => $SCHEMA_VERSION,
	description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
	app_methods => {
		'MT::App::CMS' => {
			'remove_all_entry_archive' => \&_remove_all_entry_archive,
			'update_permalink' => \&_update_permalink
		},
	},
	settings => new MT::PluginSettings([
		['rebuild_at_fview', { Default => 1 }],
		['base_url'],['base_pth'],['error_tmpl']
	]),
	system_config_template => 'sys_config.tmpl',
	blog_config_template => 'blog_config.tmpl',
	l10n_class => 'RebuildAt1stView::L10N',
	config_link => '../../'.MT->config('AdminScript').'?__mode=update_permalink',
    object_classes => [
        'MT::Permalink',
    ],
});

MT->add_plugin($plugin);

my $mt_version;
my $transform_tmpl;
my $div_success;
my $div_error;
my $v = MT->version_id; 
#->4.0 Beta 1 
if ($v =~ /^4/) { 
	$mt_version = 40;
	$transform_tmpl = 'V4.tmpl';
	$div_success = '<div id="saved" class="msg msg-success">';
	$div_error = '<div id="generic-error" class="msg msg-error">';
} else {
	$mt_version = 30;
	$transform_tmpl = 'V3.tmpl';
	$div_success = '<div class="message">';
	$div_error = '<div class="error-message">';
}

MT->add_plugin_action(	'blog',
							'../../'.MT->config('AdminScript').'?__mode=remove_all_entry_archive',
							$plugin->translate('Remove All Entry Cache')
						);

MT->add_plugin_action(	'list_entries',
							'../../'.MT->config('AdminScript').'?__mode=remove_all_entry_archive',
							$plugin->translate('Remove All Entry Cache')
						);

MT->add_plugin_action(	'list_entry',
							'../../'.MT->config('AdminScript').'?__mode=remove_all_entry_archive',
							$plugin->translate('Remove All Entry Cache')
						);

####################################### Callbacks ######################################

MT->add_callback('CMSPostSave.entry', 10, $plugin, \&_post_save_entry);
MT->add_callback('CMSPreSave_comment', 10, $plugin, \&_pre_save_comment);
MT->add_callback('CMSPreSave_ping', 10, $plugin, \&_pre_save_ping);
MT->add_callback('CMSPostDelete_entry', 10, $plugin, \&_delete_entry);
MT->add_callback('CMSPostDelete_comment', 10, $plugin, \&_delete_comment);
MT->add_callback('CMSPostDelete_ping', 10, $plugin, \&_delete_ping);

######################################### Main #########################################

sub _post_save_entry {
	my ($eh, $app, $entry, $original) = @_;
	if (&plugin_active($app)) {
		$app->publisher->remove_entry_archive_file(Entry => $entry, ArchiveType => 'Individual');
		my $next = $entry->next;
		if (defined $next) {
			$app->publisher->remove_entry_archive_file(Entry => $next, ArchiveType => 'Individual');
		}
		my $prev = $entry->previous;
		if (defined $prev) {
			$app->publisher->remove_entry_archive_file(Entry => $prev, ArchiveType => 'Individual');
		}
		my $permalink = MT::Permalink->load({id => $entry->id});
		if ($entry->status != 2) {
			if (defined $permalink) {
				$permalink->remove or die "Removing permalink failed: ", $permalink->errstr;
			}
		} else {
			if (defined $permalink) {
				$permalink->permalink($entry->permalink);
				$permalink->modified_on($entry->modified_on);
				$permalink->save or die "Saving permalink failed: ", $permalink->errstr;
			} else {
				$permalink = MT::Permalink->new;
				$permalink->id($entry->id);
				$permalink->blog_id($entry->blog_id);
				$permalink->permalink($entry->permalink);
				$permalink->modified_on($entry->modified_on);
				$permalink->save or die "Saving permalink failed: ", $permalink->errstr;
			}
		}
	}
1;
}

sub _pre_save_comment {
	my ($eh, $app, $comment) = @_;
	if (&plugin_active($app)) {
		my $entry = MT::Entry->load({id => $comment->entry_id}); 
		$app->publisher->remove_entry_archive_file(Entry => $entry, ArchiveType => 'Individual');
	}
1;
}

sub _pre_save_ping {
	my ($eh, $app, $tbping) = @_;
	if (&plugin_active($app)) {
		my $tb_id = $tbping->tb_id;
		my $trackback = MT::Trackback->load({ id => $tb_id });
		my $entry = MT::Entry->load({id => $trackback->entry_id});
		$app->publisher->remove_entry_archive_file(Entry => $entry, ArchiveType => 'Individual');
	}
1;
}

sub _delete_entry {
	my ($eh, $app, $entry) = @_;
	if (&plugin_active($app)) {
		my $permalink = MT::Permalink->load({id => $entry->id});
		if (defined $permalink) {
			$permalink->remove or die "Removing permalink failed: ", $permalink->errstr;
		}
		$app->publisher->remove_entry_archive_file(Entry => $entry, ArchiveType => 'Individual');
		my $next = $entry->next;
		if (defined $next) {
			$app->publisher->remove_entry_archive_file(Entry => $next, ArchiveType => 'Individual');
		}
		my $prev = $entry->previous;
		if (defined $prev) {
			$app->publisher->remove_entry_archive_file(Entry => $prev, ArchiveType => 'Individual');
		}
	}
1;
}

sub _delete_comment {
	my ($eh, $app, $comment) = @_;
	if (&plugin_active($app)) {
		my $entry = MT::Entry->load({id => $comment->entry_id}); 
		$app->publisher->remove_entry_archive_file(Entry => $entry, ArchiveType => 'Individual');
	}
1;
}

sub _delete_ping {
	my ($eh, $app, $tbping) = @_;
	if (&plugin_active($app)) {
		my $tb_id = $tbping->tb_id;
		my $trackback = MT::Trackback->load({ id => $tb_id });
		my $entry = MT::Entry->load({id => $trackback->entry_id});
		$app->publisher->remove_entry_archive_file(Entry => $entry, ArchiveType => 'Individual');
	}
1;
}

######################################## Actions #######################################

sub _update_permalink {
	my $app = shift;
	my $user = $app->user;
	my %param;
	$param{page_title} = $plugin->translate('Update "mt_permalink"');
	if ($user->is_superuser) {
		MT::Permalink->remove_all;
		$app->{plugin_template_path} = 'plugins/RebuildAt1stView/tmpl';
		my $b_iter = MT::Blog->load_iter(undef, { sort => 'id' });
		while (my $blog =$b_iter->()) {
			my $blog_id = $blog->id;
			my $get_from = 'blog:'.$blog_id;
			my $plugin_active = $plugin->get_config_value('rebuild_at_fview', $get_from);
			if ($plugin_active) {
				my $iter = MT::Entry->load_iter({
					blog_id => $blog_id},{
					sort => 'modified_on',
					direction => 'descend',
				});
				if (defined $iter) {
					while (my $entry = $iter->()) {
						my $permalink = MT::Permalink->new;
						$permalink->id($entry->id);
						$permalink->blog_id($entry->blog_id);
						$permalink->permalink($entry->permalink);
						$permalink->modified_on($entry->modified_on);
						$permalink->save or die "Saving permalink failed: ", $permalink->errstr;
					}
				}
			}
		}
		$param{div} = $div_success;
		$param{content} = $plugin->translate('Entry Archives List was Updated.');
		return $app->build_page($transform_tmpl, \%param);
	} else {
		$param{div} = $div_error;
		$param{content} = $plugin->translate('Permission Denied.');
		return $app->build_page($transform_tmpl, \%param);
	}
}

sub _remove_all_entry_archive {
	my $app = shift;
	my $user = $app->user;
	my $blog_id = $app->param('blog_id');
	$app->{plugin_template_path} = 'plugins/RebuildAt1stView/tmpl';
	my %param;
	$param{div} = $div_error;
	unless ($blog_id) {
		$param{content} = $plugin->translate('Please Perform This Operation by Each Blog');
		return $app->build_page($transform_tmpl, \%param);
	}
	my $blog = MT::Blog->load({id => $blog_id});
	$param{page_title} = $blog->name;
	if ($user->is_superuser) {
		my $get_from = 'blog:'.$blog_id;
		my $plugin_active = $plugin->get_config_value('rebuild_at_fview', $get_from);
		if ($plugin_active) {
			my $iter = MT::Entry->load_iter({
				blog_id => $blog_id},{
				sort => 'modified_on',
				direction => 'descend',
			});
			if (defined $iter) {
				while (my $entry = $iter->()) {
					$app->publisher->remove_entry_archive_file(Entry => $entry, ArchiveType => 'Individual');
				}
				$param{div} = $div_success;
				$param{content} = $plugin->translate('Entry Archives ware Deleted.');
				return $app->build_page($transform_tmpl, \%param);
			} else {
				$param{content} = $plugin->translate('Entry Archive was Not Found.');
				return $app->build_page($transform_tmpl, \%param);
			}
		} else {
			$param{content} = $plugin->translate('Plugin Deactive.');
			return $app->build_page($transform_tmpl, \%param);
		}
	} else {
		$param{content} = $plugin->translate('Permission Denied.');
		return $app->build_page($transform_tmpl, \%param);
	}
}

######################################### Subs #########################################

sub plugin_active {
	my $app = shift;
	my $blog = $app->blog;
	my $blog_id = $blog->id;
	my $get_from = 'blog:'.$blog_id;
	return $plugin->get_config_value('rebuild_at_fview', $get_from);
}

1;