package MTViewCGI;

use strict;
use MT;
use MT::App;
use MT::Blog;
use MT::Entry;
use MT::Builder;
use MT::Template;
use MT::TemplateMap;
use MT::Template::Context;
use MT::Permalink;
use MT::PluginData;
use HTTP::Date;

@MTViewCGI::ISA = qw(MT::App);

sub init_request {
	my $app = shift;
	$app->SUPER::init_request(@_);
	$app->add_methods( MTViewCGI => \&_MTViewCGI );
	$app->{default_mode} = 'MTViewCGI';
	$app->{requires_login} = 0;
	$app;
}

my $mt = MT->new(Config => '../../mt-config.cgi');
my $plugin = new MT::Plugin::RebuildAt1stView({ name => 'RebuildAt1stView' });
my $base_url = $plugin->get_config_value('base_url');
my $base_pth = $plugin->get_config_value('base_pth');
my $error_tmpl = $plugin->get_config_value('error_tmpl');

sub _MTViewCGI {
	my $app = shift;
	my $q = $app->param;
	my $request = $ENV{"REDIRECT_URL"};
	my $url = $base_url.$request;
	my $permalink = MT::Permalink->load({permalink => $url});
	if (defined $permalink) {
		my $blog_id = $permalink->blog_id;
		my $modified_on = $permalink->modified_on;
		my $yymmdd = substr($modified_on, 0, 8);
		my $hhmmss = substr($modified_on, 8, 8);
		my $yymmddhhmm = substr($modified_on, 0, 12);
		my $ss = substr($modified_on, 12, 2);
		$modified_on = $yymmdd.'T'.$hhmmss.'Z';
		my $touch = 'touch -t '.$yymmddhhmm.'.'.$ss.' ';
		my $ua = $ENV{'HTTP_USER_AGENT'};
		if (&is_robot($ua)) {
			$modified_on = &str2time($modified_on);
			my $http_if_mod = $ENV{'HTTP_IF_MODIFIED_SINCE'};
			$http_if_mod = &str2time($http_if_mod);
			if ($http_if_mod && ($http_if_mod >= $modified_on)) {
				print "Status: 304 Not Modified\n";
				print "Last-Modified: $modified_on\n\n";
				exit;
			}
		}
		my $entry = MT::Entry->load({id => $permalink->id});
		if (defined $entry) {
			my $tmap = MT::TemplateMap->load(
									{	blog_id => $blog_id,
										archive_type => 'Individual',
										is_preferred => 1
									},);
			my $blog = MT::Blog->load({ id => $blog_id });
			my $template = MT::Template->load({id => $tmap->template_id});
			my $page_tmpl;
			$page_tmpl = $template->text;
			my $ctx = MT::Template::Context->new;
			$ctx->stash('entry', $entry);
			$ctx->stash('blog', $blog);
			$ctx->stash('blog_id', $blog_id);
			my $build = MT::Builder->new;
			my $tokens = $build->compile($ctx, $page_tmpl)
				or return $app->error($app->translate(
					"Parse error: [_1]", $build->errstr));
			defined(my $html = $build->build($ctx, $tokens))
				or return $app->error($app->translate(
					"Build error: [_1]", $build->errstr));
			if ($html) {
				$modified_on = &time2str($modified_on);
				my $len = length($html);
				$app->set_header('Status', '200 OK');
				$app->set_header('Last-Modified', $modified_on);
				$app->set_header('Content-Length', $len);
				my $buildfile = $base_pth.$request;
				$touch .= $buildfile;
				open (OUT,">$buildfile");
				print OUT $html;
				close(OUT);
				my $res = system($touch);
				return $html;
			}
		}
	} else {
		$app->set_header('Status', '404 Not Found');
		my $error_pth = $base_pth.$error_tmpl;
		if (-f $error_pth) {
			open(FH, $error_pth) or die;
			my $template;
			while (<FH>) {
 				$template .= $_;
			}
			close(FH);
			return $template;
		} else {
			return '404 File Not Found.';
		}
	}
}

sub is_robot {
	my $ua = shift;
	if (($ua =~ /Googlebot/) || ($ua =~ /Yahoo\!\sSlurp/) || ($ua =~ /msnbot/)) {
		return 1;
	}
	return 0;
}

1;
